class Ball {
  
  float x, y;     //Ball position
  float rad;      //Ball radius
  float vx;       //Ball velocity
  float vy;
  float mass;     //The mass of each ball
  color clr;
  
  Ball( Ball toCopy) {
    x = toCopy.x;
    y = toCopy.y;
    vx = toCopy.vx;
    vy = toCopy.vy;
    rad = toCopy.rad;
    mass = toCopy.mass;
    clr = toCopy.clr;
  }
  
  Ball(float xi, float yi, float vxi, float r ) {
    x = xi;
    y = yi;
    vx = vxi;
    vy = 0;
    rad = r;
    mass = r * 0.1;
    //int i = int(random( 150, 255 ));
    //int j = int(random( 100, 255 ));
    //int k = int(random( 0, 25 ));
    clr = color( 200, 0, 0 );
  }
  
  void motion() {
    vy += gravity;
    x += vx;
    y += vy;
    //check collision with sides
    if (x + rad > width) {
      x = width - rad;
      vx *= -0.5;
    } else if (x - rad < 0) {
      x = rad;
      vx *= -0.5;
    }
    //check collision with top and bottom
    if (y + rad > height) {
      y = height - rad;
      vy *= -0.7;
    } else if (y - rad < 0) {
      y = rad;
      vy *= -0.7;
    }
    //factor to simulate friction with the floor
    if( y+rad >= height-1) {
      if( vx > 0 ) {
        vx -= friction;
      }else if( vx < 0 ) {
        vx += friction;
      }else if( vx > -0.01 || vx < 0.01 ) {
        vx = 0;
      }
    }
  }// End function Ball.motion
  
  float distance( Ball target ) {
    //finds the distance between this ball and another
    float dx = target.x - this.x;
    float dy = target.y - this.y;
    float distance = sqrt(dx*dx + dy*dy);
    return distance;
  }// End function Ball.distance
  
  float angle( Ball target ) {
    //finds the angle between this ball and another
    float dx = target.x - this.x;
    float dy = target.y - this.y;
    float angle = atan2( dy, dx );
    return angle;
  }// End function Ball.angle
  
  void display() {
    //draw the ball in the window
    noStroke();
    fill( clr );
    ellipse(x, y, rad*2, rad*2);
  }// End function Ball.display
  
  void dispVel() {
    //display a velocity vector on a ball
    //multiply the magnitude to get a larger vector for easier viewing
    stroke( 0 );
    line( x, y, x+vx*3, y+vy*3 );
  }
  
  float pdist( Point2d p ) {
    //finds the distance between this ball and another
    float dx = p.x - this.x;
    float dy = p.y - this.y;
    float distance = sqrt(dx*dx + dy*dy);
    return distance;
  }// End function Ball.distance
  
  void collide( Ball[] balls, int index, int objects ) {
    Ball temp1 = new Ball(this);
    temp1.motion();
    //only need to check each ball against balls with higher indexes
    for( int i = index; i < objects; i++) {
      Ball temp2 = new Ball(balls[i]);
      temp2.motion();
      //calculate the distance to ith ball
      float distance = temp1.distance( temp2 );
      float minDist = temp2.rad + temp1.rad;
    
      //if distance <= minDist a collision occurs
      if( distance <= minDist ) {
        float m1 = this.mass;
        float m2 = balls[i].mass;
      
        float angle = temp1.angle( temp2 );
        float sinAng = sin(angle);
        float cosAng = cos(angle);
      

        //resolve velocity components on and perpendicular to angle
        //vTan1 & vPerp1 relate to balls[index], vTan2 & vPerp2 relate to balls[i]
        float vTan1 = cosAng * this.vx + sinAng * this.vy;
        float vPerp1 = cosAng * this.vy - sinAng * this.vx;
        float vTan2 = cosAng * balls[i].vx + sinAng * balls[i].vy;
        float vPerp2 = cosAng * balls[i].vy - sinAng * balls [i].vx;
      
        //calc velocity for ball at index
        float vFinTan1 = ((m1 - m2) * vTan1 + 2 * m2 * vTan2) / (m1 + m2);
        float vFinPerp1 = vPerp1;
        //calc velocity for ball at i
        float vFinTan2 = ( 2 * m1 * vTan1 + (m2 - m1) * vTan2 ) / (m1 + m2);
        float vFinPerp2 = vPerp2;
      
        //update velocities remember to project back to normal coordinate system
        this.vx = cosAng * vFinTan1 - sinAng * vFinPerp1;
        this.vy = cosAng * vFinPerp1 + sinAng * vFinTan1;
        balls[i].vx = cosAng * vFinTan2 - sinAng * vFinPerp2;
        balls[i].vy = cosAng * vFinPerp2 + sinAng * vFinTan2;
      }
    }//End for loop
  }// End function collide

  void pcollide( Point2d[] pnts ) {
    Ball tempBall = new Ball( this );
    tempBall.motion();
    int closesti = 0;
    float d2i = this.rad+1;
    for( int i = 0; i < numPoints; i++ ) {
      float tempd = tempBall.pdist( pnts[i] );
      if( tempd <= d2i ) {
        closesti = i;
        d2i = tempd;
      }
    }
    
    if( d2i <= this.rad ) {
      //calculate a normal vector 
      int surf1; int surf2;
      surf1 = (closesti +1) % pnts.length;
      if( closesti == 0 ) {
        surf2 = pnts.length-1;
      }else {
        surf2 = (closesti -1) % pnts.length;
      }
      
      float srfAng = pnts[surf1].angle( pnts[surf2] );
      float Ang2 = pnts[closesti].angle( pnts[surf2] );
      float Ang1 = pnts[closesti].angle( pnts[surf1] );
      float srfNrm;
      if( Ang1 == PI ) {
        Ang1 = -PI;
      }
      if( Ang2 == -PI ) {
        Ang2 = PI;
      }
      if( Ang2 >= Ang1 ){
        srfNrm = srfAng + PI/2;
      }else {
        srfNrm = srfAng - PI/2;
      }
      
      float tempAng = srfNrm - PI/2;
      float cosAng = cos(tempAng);
      float sinAng = sin(tempAng);
      
      //resolve components
      float vi = cosAng * this.vx + sinAng * this.vy;
      float vj = cosAng * this.vy - sinAng * this.vx;
      
      vj = -e*vj;
      
      this.vx = vi*cosAng - vj* sinAng;
      this.vy = vi*sinAng + vj* cosAng;
      
      if( this.vy >= -gravity && this.vy < 0 ) {
        if ( this.vy == -gravity*e ){
          this.y -= gravity;
        } else {
          while ( this.pdist( pnts[closesti] ) < this.rad ) {
            this.y -= 1;
          }
        }
        this.vy = 0;
      }
    }//End if
  
  }// End function collide

}// End class Ball
//----------------------------------------------------------------------
